#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import math, tempfile, copy, cPickle, inkex, simplepath

def findend(d):
    end = []
    subPathStart = []
    for cmd,params in d:
        if cmd == 'M':
            subPathStart = params[-2:]
        if cmd == 'Z':
            end = subPathStart[:]
        else:
            end = params[-2:]
    return end

class Kochify(inkex.Effect):
    def effect(self):
        try:
            f = open(tempfile.gettempdir() + '/kochify.bin', 'r')
            proto = cPickle.load(f)
            f.close()                
        except:
            return False
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                d = node.attributes.getNamedItem('d')
                p = simplepath.parsePath(d.value)
                last = p[0][1][-2:]
                subPathStart = []
                cur = []
                new = []
                for i in range(len(p)):
                    rep = copy.deepcopy(proto['path'])
                    cmd, params = p[i]
                    if cmd == 'M':
                        subPathStart = params[-2:]
                        new.append(copy.deepcopy(p[i]))
                        continue
                    if cmd == 'Z':
                        cur = subPathStart[:]
                    else:
                        cur = params[-2:]

                    if last == cur:
                        continue

                    dx = cur[0]-last[0]
                    dy = cur[1]-last[1]
                    length = math.sqrt((dx**2) + (dy**2))
                    angle = math.atan2(dy,dx)
        
                    scale = length / proto['length']
                    rotate = angle - proto['angle']
                    simplepath.scalePath(rep,scale,scale)
                    simplepath.rotatePath(rep, rotate)
                    repend = findend(rep)
                    transx = cur[0] - repend[0]
                    transy = cur[1] - repend[1]
                    simplepath.translatePath(rep, transx, transy)
                    
                    if proto['endsinz']:
                        new.extend(rep[:])
                    else:
                        new.extend(rep[1:])
                    last = cur[:]
                
                d.value = simplepath.formatPath(new)
e = Kochify()
e.affect()



#!/usr/bin/env python 
"""
svg_and_media_zip_output.py
An extention which collects all images to the documents directory and 
creates a zip archive containing all images and the document

Copyright (C) 2005 Pim Snel, pim@lingewoud.com
this is  the first Python script  ever created
its based on embedimage.py

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Version 0.3

TODO
- fix bug: not saving existing .zip after a Collect for Output is run
     this bug occurs because after running an effect extention the inkscape:output_extension is reset to svg.inkscape
     the file name is still xxx.zip. after saving again the file xxx.zip is written with a plain .svg which 
     looks like a corrupt zip
- maybe add better extention
"""

import inkex, os.path
import os
import string
import zipfile
import shutil
import sys
import tempfile

class MyEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)

        self.documentDst=None

    def parseTmp(self,file=None):
        """Parse document in specified file or on stdin"""
        reader = inkex.xml.dom.ext.reader.Sax2.Reader()
        try:
            try:
                stream = open(file,'r')
            except:
                stream = open(self.args[-1],'r')
        except:
            stream = sys.stdin
        self.documentDst = reader.fromStream(stream)
        stream.close()

    def output(self):
        pass

    def effect(self):
        
        #get needed info from orig document
        ctx_orig = inkex.xml.xpath.Context.Context(self.document,processorNss=inkex.NSS)

        ttmp_orig = inkex.xml.xpath.Evaluate('/svg',self.document, context=ctx_orig)
        
        docbase = ttmp_orig[0].attributes.getNamedItemNS(inkex.NSS[u'sodipodi'],'docbase')
        docname = ttmp_orig[0].attributes.getNamedItemNS(inkex.NSS[u'sodipodi'],'docname')

        orig_tmpfile = sys.argv[1]

        # create destination zip in same directory as the document
        z = zipfile.ZipFile(docbase.value + '/'+ docname.value + '.zip', 'w')    

        #create os temp dir
        tmp_dir = tempfile.mkdtemp()

        #fixme replace whatever extention
        docstripped = docname.value.replace('.zip', '')
    
        #read tmpdoc and copy all images to temp dir
        for node in inkex.xml.xpath.Evaluate('//image',self.document, context=ctx_orig):
            self.collectAndZipImages(node, tmp_dir, docname, z)

        ##copy tmpdoc to tempdir
        dst_file = os.path.join(tmp_dir, docstripped)
        stream = open(dst_file,'w')
    
        inkex.xml.dom.ext.Print(self.document,stream)
        
        stream.close()
        
        z.write(dst_file.encode("latin-1"),docstripped.encode("latin-1")+'.svg') 
        z.close()

        shutil.move(docbase.value + '/'+ docname.value + '.zip',docbase.value + '/'+ docname.value)
        
        shutil.rmtree(tmp_dir)

    def collectAndZipImages(self, node, tmp_dir, docname, z):
        xlink = node.attributes.getNamedItemNS(inkex.NSS[u'xlink'],'href')
        if (xlink.value[:4]!='data'):
            absref=node.attributes.getNamedItemNS(inkex.NSS[u'sodipodi'],'absref')

            if (os.path.isfile(absref.value)):
                shutil.copy(absref.value,tmp_dir)
                z.write(absref.value.encode("latin-1"),os.path.basename(absref.value).encode("latin-1"))

            elif (os.path.isfile(tmp_dir + '/' + absref.value)):
                shutil.copy(tmp_dir + '/' + absref.value,tmp_dir)
                z.write(tmp_dir + '/' + absref.value.encode("latin-1"),os.path.basename(absref.value).encode("latin-1"))

            xlink.value = os.path.basename(absref.value)
            absref.value = os.path.basename(absref.value)

            
e = MyEffect()
e.affect()



#!/usr/bin/env python
'''
    Copyright (C) 2004 Aaron Cyril Spike

    This file is part of FretFind 2-D.

    FretFind 2-D is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FretFind 2-D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FretFind 2-D; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import sys
from ffgeom import *
threshold=0.0000000001

def FindFrets(strings, meta, scale, tuning, numfrets):
    scale = scale['steps']

    #if the string ends don't fall on the nut and bridge
    #don't look for partial frets.
    numStrings = len(strings)
    doPartials = True
    parallelFrets = True
    
    nut = Segment(strings[0][0],strings[-1][0])
    bridge = Segment(strings[0][1],strings[-1][1])
    midline = Segment(
        Point((nut[1]['x']+nut[0]['x'])/2.0,(nut[1]['y']+nut[0]['y'])/2.0),
        Point((bridge[1]['x']+bridge[0]['x'])/2.0,(bridge[1]['y']+bridge[0]['y'])/2.0))
    for s in strings:
        if nut.perpDistanceToPoint(s[0])>=threshold or bridge.perpDistanceToPoint(s[1])>=threshold:
            doPartials = False
            break

    denom = ((bridge[1]['y']-bridge[0]['y'])*(nut[1]['x']-nut[0]['x']))-((bridge[1]['x']-bridge[0]['x'])*(nut[1]['y']-nut[0]['y']))
    if denom != 0:
        parallelFrets = False

    fretboard = []
    tones = len(scale)-1
    for i in range(len(strings)):
        base = tuning[i]
        frets = []
        if doPartials:
            frets.append(Segment(meta[i][0],meta[i+1][0]))
        else:
            frets.append(Segment(strings[i][0],strings[i][0]))
        last = strings[i][0]

        for j in range(numfrets):
            step=((base+j-1)%(tones))+1
            ratio=1.0-((scale[step][1]*scale[step-1][0])/(scale[step][0]*scale[step-1][1]))
            x = last['x']+(ratio*(strings[i][1]['x']-last['x']))
            y = last['y']+(ratio*(strings[i][1]['y']-last['y']))
            current = Point(x,y)    
            temp = Segment(strings[i][0],current)
            totalRatio = temp.length()/strings[i].length()
            
            if doPartials:
                #partials depending on outer strings (questionable)
                if parallelFrets:
                    temp = nut.createParallel(current)
                else:
                    temp = Segment(strings[0].pointAtLength(strings[0].length()*totalRatio),
                        strings[-1].pointAtLength(strings[-1].length()*totalRatio))
                frets.append(Segment(intersectSegments(temp,meta[i]),intersectSegments(temp,meta[i+1])))
            else:
                frets.append(Segment(current,current))
            last = current
        fretboard.append(frets)
    return fretboard
    
def FindStringsSingleScale(numStrings,scaleLength,nutWidth,bridgeWidth,oNF,oBF,oNL,oBL):
    strings = []
    meta = []
    nutHalf = nutWidth/2
    bridgeHalf = bridgeWidth/2
    nutCandidateCenter = (nutHalf) + oNL
    bridgeCandidateCenter = (bridgeHalf) + oBL
    if bridgeCandidateCenter >= nutCandidateCenter:
        center = bridgeCandidateCenter
    else:
        center = nutCandidateCenter
    nutStringSpacing = nutWidth/(numStrings-1)
    bridgeStringSpacing = bridgeWidth/(numStrings-1)
    
    for i in range(numStrings):
        strings.append(Segment(Point(center+nutHalf-(i*nutStringSpacing),0),
            Point(center+bridgeHalf-(i*bridgeStringSpacing),scaleLength)))

    meta.append(Segment(Point(center+nutHalf+oNF,0),Point(center+bridgeHalf+oBF,scaleLength)))
    for i in range(1,numStrings):
        meta.append(Segment(
            Point((strings[i-1][0]['x']+strings[i][0]['x'])/2.0,
                (strings[i-1][0]['y']+strings[i][0]['y'])/2.0),
            Point((strings[i-1][1]['x']+strings[i][1]['x'])/2.0,
                (strings[i-1][1]['y']+strings[i][1]['y'])/2.0)))
    meta.append(Segment(Point(center-(nutHalf+oNL),0),Point(center-(bridgeHalf+oBL),scaleLength)))

    return strings, meta

def FindStringsMultiScale(numStrings,scaleLengthF,scaleLengthL,nutWidth,bridgeWidth,perp,oNF,oBF,oNL,oBL):
    strings = []
    meta = []
    nutHalf = nutWidth/2
    bridgeHalf = bridgeWidth/2
    nutCandidateCenter = (nutHalf)+oNL
    bridgeCandidateCenter = (bridgeHalf)+oBL
    if bridgeCandidateCenter >= nutCandidateCenter:
        xcenter = bridgeCandidateCenter
    else:
        nutCandidateCenter

    fbnxf = xcenter+nutHalf+oNF
    fbbxf = xcenter+bridgeHalf+oBF
    fbnxl = xcenter-(nutHalf+oNL)
    fbbxl = xcenter-(bridgeHalf+oBL)

    snxf = xcenter+nutHalf
    sbxf = xcenter+bridgeHalf
    snxl = xcenter-nutHalf
    sbxl = xcenter-bridgeHalf

    fdeltax = sbxf-snxf
    ldeltax = sbxl-snxl
    fdeltay = math.sqrt((scaleLengthF*scaleLengthF)-(fdeltax*fdeltax))
    ldeltay = math.sqrt((scaleLengthL*scaleLengthL)-(ldeltax*ldeltax))

    fperp = perp*fdeltay
    lperp = perp*ldeltay

    #temporarily place first and last strings
    first = Segment(Point(snxf,0),Point(sbxf,fdeltay))
    last = Segment(Point(snxl,0),Point(sbxl,ldeltay))
    
    if fdeltay<=ldeltay:
        first.translate(0,(lperp-fperp))
    else:
        last.translate(0,(fperp-lperp))

    nut = Segment(first[0].copy(),last[0].copy())
    bridge = Segment(first[1].copy(),last[1].copy())
    #overhang measurements are now converted from delta x to along line lengths
    oNF = (oNF*nut.length())/nutWidth
    oNL = (oNL*nut.length())/nutWidth
    oBF = (oBF*bridge.length())/bridgeWidth
    oBL = (oBL*bridge.length())/bridgeWidth
    #place fretboard edges
    fbf = Segment(nut.pointAtLength(-oNF),bridge.pointAtLength(-oBF))
    fbl = Segment(nut.pointAtLength(nut.length()+oNL),bridge.pointAtLength(bridge.length()+oBL))
    #normalize values into the first quadrant via translate
    if fbf[0]['y']<0 or fbl[0]['y']<0:
        if fbf[0]['y']<=fbl[0]['y']:
            move = -fbf[0]['y']
        else:
            move = -fbl[0]['y']
        
        first.translate(0,move)
        last.translate(0,move)
        nut.translate(0,move)
        bridge.translate(0,move)
        fbf.translate(0,move)
        fbl.translate(0,move)

    #output values
    nutStringSpacing = nut.length()/(numStrings-1)
    bridgeStringSpacing = bridge.length()/(numStrings-1)
    strings.append(first)
    for i in range(1,numStrings-1):
        n = nut.pointAtLength(i*nutStringSpacing)
        b = bridge.pointAtLength(i*bridgeStringSpacing)
        strings.append(Segment(Point(n['x'],n['y']),Point(b['x'],b['y'])))
    strings.append(last)

    meta.append(fbf)
    for i in range(1,numStrings):
        meta.append(Segment(
            Point((strings[i-1][0]['x']+strings[i][0]['x'])/2.0,
                (strings[i-1][0]['y']+strings[i][0]['y'])/2.0),
            Point((strings[i-1][1]['x']+strings[i][1]['x'])/2.0,
                (strings[i-1][1]['y']+strings[i][1]['y'])/2.0)))
    
    meta.append(fbl)
    
    return strings, meta



#!/usr/bin/env python 
'''
Copyright (C) 2006 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex
import sys, os, tempfile

class MyEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
    def output(self):
        pass
    def effect(self):
        svg_file = self.args[-1]
        node = inkex.xml.xpath.Evaluate('/svg',self.document)[0]
        docname = node.attributes.getNamedItemNS(inkex.NSS[u'sodipodi'],'docname').value[:-4]

        #create os temp dir
        tmp_dir = tempfile.mkdtemp()

        area = '--export-area-canvas'
        pngs = []
        path = "/svg/*[name()='g' or @style][@id]"
        for node in inkex.xml.xpath.Evaluate(path,self.document):
            id = node.attributes.getNamedItem('id').value
            name = "%s.png" % id
            filename = os.path.join(tmp_dir, name)
            command = "inkscape -i %s -j %s -e %s %s " % (id, area, filename, svg_file)
            f = os.popen(command,'r')
            f.read()
            f.close()
            pngs.append(filename)

        filelist = '"%s"' % '" "'.join(pngs)
        xcf = os.path.join(tmp_dir, "%s.xcf" % docname)
        script_fu = """
(define
  (png-to-layer img png_filename)
  (let*
    (
      (png (car (file-png-load RUN-NONINTERACTIVE png_filename png_filename)))
      (png_layer (car (gimp-image-get-active-layer png)))
      (xcf_layer (car (gimp-layer-new-from-drawable png_layer img)))
    )
    (gimp-image-add-layer img xcf_layer -1)
    (gimp-drawable-set-name xcf_layer png_filename)
  )
)
(let*
  (
    (img (car (gimp-image-new 200 200 RGB)))
  )
  (gimp-image-undo-disable img)
  (let* ((filelist '(%s)))
    (while filelist
      (let* ((filename (car filelist)))
        (png-to-layer img filename)
      )
      (set! filelist (cdr filelist))
    )
  )
  (gimp-image-resize-to-layers img)
  (gimp-image-undo-enable img)
  (gimp-file-save RUN-NONINTERACTIVE img (car (gimp-image-get-active-layer img)) "%s" "%s"))
(gimp-quit 0)
        """ % (filelist, xcf, xcf)

        junk = os.path.join(tmp_dir, 'junk_from_gimp.txt')
        f = os.popen('gimp -i -b - > %s' % junk,'w')
        f.write(script_fu)
        f.close()
        
        x = open(xcf, 'r')
        sys.stdout.write(x.read())
        x.close()

e = MyEffect()
e.affect()


#!/usr/bin/env python
'''
    Copyright (C) 2004 Aaron Cyril Spike

    This file is part of FretFind 2-D.

    FretFind 2-D is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FretFind 2-D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FretFind 2-D; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import math

def ETScale(tones, octave=2.0):
    octave, tones = float(octave), float(tones)
    scale = {'steps':[[1.0,1.0]], 'title':'', 'errors':0, 'errorstring':''}
    if not tones:
        scale['errors'] += 1
        scale['errorstring'] = 'Error: Number of tones must be non zero!'
    else:    
        ratio = octave**(1/tones)
        scale['title'] = '%s root of %s Equal Temperament' % (tones, octave)
        scale['steps'].append([ratio,1])
    return scale

def ScalaScale(scala):
    #initial step 0 or 1/1 is implicit
    scale = {'steps':[[1.0,1.0]], 'title':'', 'errors':0, 'errorstring':''}

    #split scale discarding commments
    lines = [l.strip() for l in scala.strip().splitlines() if not l.strip().startswith('!')]

    #first line may be blank and contains the title
    scale['title'] =  lines.pop(0)

    #second line indicates the number of note lines that should follow
    expected = int(lines.pop(0))

    #discard blank lines and anything following whitespace    
    lines = [l.split()[0] for l in lines if l != '']
    
    if len(lines) != expected:
        scale['errors'] += 1
        scale['errorstring'] = 'Error: expected %s more tones but found %s!' % (expected,len(lines))
    else:
        for l in lines:
            #interpret anyline containing a dot as cents
            if l.find('.') >= 0:
                num = 2**(float(l)/1200)
                denom = 1
            #everything else is a ratio
            elif l.find('/') >=0:
                l = l.split('/')
                num = float(int(l[0]))
                denom = float(int(l[1]))
            else:
                num = float(int(l))
                denom = 1.0
            scale['steps'].append([num,denom])
            
            if (num < 0) ^ (denom <= 0):
                scale['errors'] += 1
                scale['errorstring'] += 'Error at "'+l+'": Negative and undefined ratios are not allowed!\n'
    return scale



#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import sys
import inkex, simplestyle, simplepath
import ffscale, ffproc

def seg2path(seg):
    return "M%s,%sL%s,%s" % (seg[0]['x'],seg[0]['y'],seg[1]['x'],seg[1]['y'])

class FretFind(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.doScala = False
        self.doMultiScale = False
        self.OptionParser.add_option("--scalafile",
                        action="store", type="string", 
                        dest="scalafile", default=None,
                        help="")
        self.OptionParser.add_option("--scalascale",
                        action="store", type="string", 
                        dest="scalascale", default=None,
                        help="")
        self.OptionParser.add_option("--etbase",
                        action="store", type="float", 
                        dest="etbase", default=2.0,
                        help="")
        self.OptionParser.add_option("--etroot",
                        action="store", type="float", 
                        dest="etroot", default=12.0,
                        help="")
        self.OptionParser.add_option("--tuning",
                        action="store", type="string", 
                        dest="tuning", default="0;0;0;0;0;0",
                        help="")
        self.OptionParser.add_option("--perpdist",
                        action="store", type="float", 
                        dest="perpdist", default=0.5,
                        help="")
        self.OptionParser.add_option("--offset",
                        action="store", type="float", 
                        dest="offset", default=0.0,
                        help="")
        self.OptionParser.add_option("--scalelength",
                        action="store", type="float", 
                        dest="scalelength", default=25.0,
                        help="")
        self.OptionParser.add_option("--firstscalelength",
                        action="store", type="float", 
                        dest="firstscalelength", default=None,
                        help="")
        self.OptionParser.add_option("--lastscalelength",
                        action="store", type="float", 
                        dest="lastscalelength", default=None,
                        help="")
        self.OptionParser.add_option("--nutwidth",
                        action="store", type="float", 
                        dest="nutwidth", default=1.5,
                        help="")
        self.OptionParser.add_option("--bridgewidth",
                        action="store", type="float", 
                        dest="bridgewidth", default=2.5,
                        help="")
        self.OptionParser.add_option("--frets",
                        action="store", type="int", 
                        dest="frets", default=24,
                        help="")
        self.OptionParser.add_option("--strings",
                        action="store", type="int", 
                        dest="strings", default=6,
                        help="")
        self.OptionParser.add_option("--fbedges",
                        action="store", type="float", 
                        dest="fbedges", default=0.0975,
                        help="")
        self.OptionParser.add_option("--pxperunit",
                        action="store", type="float", 
                        dest="pxperunit", default=90,
                        help="")
    def checkopts(self):
        #scale type
        if self.options.scalascale is not None:
            self.doScala = True
        if self.options.scalafile is not None:
            if self.doScala:
                sys.stderr.write('Mutually exclusive options: if found scalafile will override scalascale.')
            else:
                self.options.scalascale = ""
            self.doScala = True
            try:
                f = open(self.options.scalafile,'r')
                self.options.scalascale = f.read()
            except: 
                sys.exit('Scala scale description file expected.')
        #scale
        if self.doScala:
            self.scale = ffscale.ScalaScale(self.options.scalascale)
        else:
            self.scale = ffscale.ETScale(self.options.etroot,self.options.etbase)
    
        #string length
        first = self.options.firstscalelength is not None
        last = self.options.lastscalelength is not None
        if first and last:
            self.doMultiScale = True
        elif first:
            sys.stderr.write('Missing lastscalelength: overriding scalelength with firstscalelength.')
            self.options.scalelength = self.options.firstscalelength
        elif last:
            sys.stderr.write('Missing firstscalelength: overriding scalelength with lastscalelength.')
            self.options.scalelength = self.options.lastscalelength
        
        #tuning
        self.tuning = [int(t.strip()) for t in self.options.tuning.split(";")]
        self.tuning.extend([0 for i in range(self.options.strings - len(self.tuning))])

    def effect(self):
        self.checkopts()

        o = self.options.fbedges
        oNF,oBF,oNL,oBL = o,o,o,o

        if self.doMultiScale:
            strings, meta = ffproc.FindStringsMultiScale(self.options.strings,self.options.firstscalelength,
                self.options.lastscalelength,self.options.nutwidth,self.options.bridgewidth,
                self.options.perpdist,oNF,oBF,oNL,oBL)
        else:
            strings, meta = ffproc.FindStringsSingleScale(self.options.strings,self.options.scalelength,
                self.options.nutwidth,self.options.bridgewidth,
                oNF,oBF,oNL,oBL)
        
        frets = ffproc.FindFrets(strings, meta, self.scale, self.tuning, self.options.frets)
        
        edgepath = seg2path(meta[0]) + seg2path(meta[-1])
        stringpath = "".join([seg2path(s) for s in strings])
        fretpath = "".join(["".join([seg2path(f) for f in s]) for s in frets])
    
        group = self.document.createElement('svg:g')
        group.setAttribute('transform',"scale(%s,%s)" % (self.options.pxperunit,self.options.pxperunit))
        self.current_layer.appendChild(group)
        
        edge = self.document.createElement('svg:path')
        s = {'stroke-linejoin': 'miter', 'stroke-width': '0.01px', 
            'stroke-opacity': '1.0', 'fill-opacity': '1.0', 
            'stroke': '#0000FF', 'stroke-linecap': 'butt', 
            'fill': 'none'}
        edge.setAttribute('style', simplestyle.formatStyle(s))
        edge.setAttribute('d', edgepath)

        string = self.document.createElement('svg:path')
        s = {'stroke-linejoin': 'miter', 'stroke-width': '0.01px', 
            'stroke-opacity': '1.0', 'fill-opacity': '1.0', 
            'stroke': '#FF0000', 'stroke-linecap': 'butt', 
            'fill': 'none'}
        string.setAttribute('style', simplestyle.formatStyle(s))
        string.setAttribute('d', stringpath)

        fret = self.document.createElement('svg:path')
        s = {'stroke-linejoin': 'miter', 'stroke-width': '0.01px', 
            'stroke-opacity': '1.0', 'fill-opacity': '1.0', 
            'stroke': '#000000', 'stroke-linecap': 'butt', 
            'fill': 'none'}
        fret.setAttribute('style', simplestyle.formatStyle(s))
        fret.setAttribute('d', fretpath)

        group.appendChild(edge)
        group.appendChild(string)
        group.appendChild(fret)

e = FretFind()
e.affect()


#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



drawwave() was translated into python from the postscript version
described at http://www.ghostscript.com/person/toby/ and located
at http://www.telegraphics.com.au/sw/sine.ps . 
http://www.tinaja.com/glib/bezsine.pdf shows another method for
approximating sine with beziers.

The orginal postscript version displayed the following copyright 
notice and was released under the terms of the GPL:
Copyright (C) 2001-3 Toby Thain, toby@telegraphics.com.au
'''
import inkex, simplepath, simplestyle
from math import *
from random import *

def drawwave(samples, periods, width, height, left, top, 
    fx = "sin(x)", fpx = "cos(x)", fponum = True):

    # step is the distance between nodes on x
    step = 2*pi / samples
    third = step / 3.0
    
    # coords and scales based on the source rect
    xoff = left
    yoff = top + (height / 2)
    scalex = width / (2*pi * periods)
    scaley = height / 2
    procx = lambda x: x * scalex + xoff
    procy = lambda y: y * scaley + yoff

    # functions specified by the user
    if fx != "":
        f = eval('lambda x: ' + fx)
    if fpx != "":
        fp = eval('lambda x: ' + fpx)

    # initialize function and derivative for 0;
    # they are carried over from one iteration to the next, to avoid extra function calculations         
    y0 = f(0) 
    if fponum == True: # numerical derivative, using 0.001*step as the small differential
        d0 = (f(0 + 0.001*step) - y0)/(0.001*step)
    else: # derivative given by the user
        d0 = fp(0)

    a = [] # path array 
    a.append(['M',[procx(0.0), procy(y0)]]) # initial moveto

    for i in range(int(samples * periods)):
        x = i * step 
        y1 = f(x + step)
        if fponum == True: # numerical derivative
            d1 = (y1 - f(x + step - 0.001*step))/(0.001*step)
        else: # derivative given by the user
            d1 = fp(x + step)
        # create curve
        a.append(['C',[procx(x + third), procy(y0 + (d0 * third)), 
            procx(x + (step - third)), procy(y1 - (d1 * third)),
            procx(x + step), procy(y1)]])
        y0 = y1 # next segment's y0 is this segment's y1
        d0 = d1 # we assume the function is smooth everywhere, so carry over the derivative too
            
    return a

class Wavy(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-p", "--periods",
                        action="store", type="float", 
                        dest="periods", default=4.0,
                        help="Periods (2*Pi each)")
        self.OptionParser.add_option("-s", "--samples",
                        action="store", type="int", 
                        dest="samples", default=8,
                        help="Samples per period")    
        self.OptionParser.add_option("--fofx",
                        action="store", type="string", 
                        dest="fofx", default="sin(x)",
                        help="f(x) for plotting")    
        self.OptionParser.add_option("--fponum",
                        action="store", type="inkbool", 
                        dest="fponum", default=True,
                        help="Calculate the first derivative numerically")    
        self.OptionParser.add_option("--fpofx",
                        action="store", type="string", 
                        dest="fpofx", default="cos(x)",
                        help="f'(x) for plotting")    
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'rect':
                new = self.document.createElement('svg:path')
                x = float(node.attributes.getNamedItem('x').value)
                y = float(node.attributes.getNamedItem('y').value)
                w = float(node.attributes.getNamedItem('width').value)
                h = float(node.attributes.getNamedItem('height').value)

                s = node.attributes.getNamedItem('style').value
                new.setAttribute('style', s)
                try:
                    t = node.attributes.getNamedItem('transform').value
                    new.setAttribute('transform', t)
                except AttributeError:
                    pass
                new.setAttribute('d', simplepath.formatPath(
                            drawwave(self.options.samples, 
                                self.options.periods,
                                w,h,x,y,
                                self.options.fofx, 
                                self.options.fpofx,
                                self.options.fponum)))
                node.parentNode.appendChild(new)
                node.parentNode.removeChild(node)

e = Wavy()
e.affect()



#!/usr/bin/env python
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import math
class pTurtle:
    '''A Python path turtle'''
    def __init__(self, home=(0,0)):
        self.__home = [home[0], home[1]]
        self.__pos = self.__home[:]
        self.__heading = -90
        self.__path = ""
        self.__draw = True
        self.__new = True
    def forward(self,mag):
        self.setpos((self.__pos[0] + math.cos(math.radians(self.__heading))*mag,
            self.__pos[1] + math.sin(math.radians(self.__heading))*mag))
    def backward(self,mag):
        self.setpos((self.__pos[0] - math.cos(math.radians(self.__heading))*mag,
            self.__pos[1] - math.sin(math.radians(self.__heading))*mag))
    def right(self,deg):
        self.__heading -= deg
    def left(self,deg):
        self.__heading += deg
    def penup(self):
        self.__draw = False
        self.__new = False
    def pendown(self):
        if not self.__draw:
            self.__new = True
        self.__draw = True
    def pentoggle(self):
        if self.__draw:
            self.penup()
        else:
            self.pendown()
    def home(self):
        self.setpos(self.__home)
    def clean(self):
        self.__path = ''
    def clear(self):
        self.clean()
        self.home()
    def setpos(self,(x,y)):
        if self.__new:
            self.__path += "M"+",".join([str(i) for i in self.__pos])
            self.__new = False
        self.__pos = [x, y]
        if self.__draw:
            self.__path += "L"+",".join([str(i) for i in self.__pos])
    def getpos(self):
        return self.__pos[:]
    def setheading(self,deg):
        self.__heading = deg
    def getheading(self):
        return self.__heading
    def sethome(self,(x,y)):
        self.__home = [x, y]
    def getPath(self):
        return self.__path
    fd = forward
    bk = backward
    rt = right
    lt = left
    pu = penup
    pd = pendown



#!/usr/bin/env python 
'''
Copyright (C) 2006 Nathan Hurst
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex, simplestyle, simplepath,sys,cubicsuperpath, bezmisc

def numsegs(csp):
    return sum([len(p)-1 for p in csp])
def interpcoord(v1,v2,p):
    return v1+((v2-v1)*p)
def interppoints(p1,p2,p):
    return [interpcoord(p1[0],p2[0],p),interpcoord(p1[1],p2[1],p)]
def pointdistance((x1,y1),(x2,y2)):
    return math.sqrt(((x2 - x1) ** 2) + ((y2 - y1) ** 2))
def bezlenapprx(sp1, sp2):
    return pointdistance(sp1[1], sp1[2]) + pointdistance(sp1[2], sp2[0]) + pointdistance(sp2[0], sp2[1])
def tpoint((x1,y1), (x2,y2), t = 0.5):
    return [x1+t*(x2-x1),y1+t*(y2-y1)]
def cspbezsplit(sp1, sp2, t = 0.5):
    m1=tpoint(sp1[1],sp1[2],t)
    m2=tpoint(sp1[2],sp2[0],t)
    m3=tpoint(sp2[0],sp2[1],t)
    m4=tpoint(m1,m2,t)
    m5=tpoint(m2,m3,t)
    m=tpoint(m4,m5,t)
    return [[sp1[0][:],sp1[1][:],m1], [m4,m,m5], [m3,sp2[1][:],sp2[2][:]]]
def cspbezsplitatlength(sp1, sp2, l = 0.5, tolerance = 0.001):
    bez = (sp1[1][:],sp1[2][:],sp2[0][:],sp2[1][:])
    t = bezmisc.beziertatlength(bez, l, tolerance)
    return cspbezsplit(sp1, sp2, t)
def cspseglength(sp1,sp2, tolerance = 0.001):
    bez = (sp1[1][:],sp1[2][:],sp2[0][:],sp2[1][:])
    return bezmisc.bezierlength(bez, tolerance)    
def csplength(csp):
    total = 0
    lengths = []
    for sp in csp:
        lengths.append([])
        for i in xrange(1,len(sp)):
            l = cspseglength(sp[i-1],sp[i])
            lengths[-1].append(l)
            total += l            
    return lengths, total

class Length(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-f", "--fontsize",
                        action="store", type="string", 
                        dest="fontsize", default="20",
                        help="Size of node label numbers")    
        self.OptionParser.add_option("-o", "--offset",
                        action="store", type="string", 
                        dest="offset", default="-4",
                        help="The distance above the curve")    
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                self.group = self.document.createElement('svg:g')
                node.parentNode.appendChild(self.group)
                
                try:
                    t = node.attributes.getNamedItem('transform').value
                    self.group.setAttribute('transform', t)
                except AttributeError:
                    pass

                a =[]
                p = cubicsuperpath.parsePath(node.attributes.getNamedItem('d').value)
                num = 1
                slengths, stotal = csplength(p)
                self.addTextOnPath(self.group,0, 0,str(stotal), id, self.options.offset)


    def addTextOnPath(self,node,x,y,text, id,dy=0):
                new = self.document.createElement('svg:text')
                tp = self.document.createElement('svg:textPath')
                s = {'font-size': self.options.fontsize, 'fill-opacity': '1.0', 'stroke': 'none',
                    'font-weight': 'normal', 'font-style': 'normal', 'fill': '#000000'}
                new.setAttribute('style', simplestyle.formatStyle(s))
                new.setAttribute('x', str(x))
                new.setAttribute('y', str(y))
                tp.setAttributeNS('http://www.w3.org/1999/xlink','xlink:href', '#'+id)
                tp.setAttribute('startOffset', "50%")
                #tp.setAttribute('dy', dy) # dubious merit
                new.appendChild(tp)
                tp.appendChild(self.document.createTextNode(str(text)))
                node.appendChild(new)

e = Length()
e.affect()



#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

import inkex, os, base64

class MyEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)

    def effect(self):
        ctx = inkex.xml.xpath.Context.Context(self.document,processorNss=inkex.NSS)
        
        # if there is a selection only embed selected images
        # otherwise embed all images
        if (self.options.ids):
            for id, node in self.selected.iteritems():
                if node.tagName == 'image':
                    self.embedImage(node)
        else:
            path = '//image'
            for node in inkex.xml.xpath.Evaluate(path,self.document, context=ctx):
                self.embedImage(node)
    def embedImage(self, node):
        xlink = node.attributes.getNamedItemNS(inkex.NSS[u'xlink'],'href')
        if (xlink.value[:4]!='data'):
            absref=node.attributes.getNamedItemNS(inkex.NSS[u'sodipodi'],'absref')
            if (os.path.isfile(absref.value)):
                file = open(absref.value,"rb").read()
                embed=True
                if (file[:4]=='\x89PNG'):
                    type='image/png'
                elif (file[:2]=='\xff\xd8'):
                    type='image/jpg'
                else:
                    embed=False
                if (embed):
                    xlink.value = 'data:%s;base64,%s' % (type, base64.encodestring(file))
                    node.removeAttributeNS(inkex.NSS[u'sodipodi'],'absref')
                else:
                    inkex.debug("%s is not of type image/png or image/jpg" % absref.value)
            else:
                inkex.debug("Sorry we could not locate %s" % absref.value)
e = MyEffect()
e.affect()



#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import random, math, inkex, cubicsuperpath

def randomize((x, y), r, norm):
    if norm:
        r = abs(random.normalvariate(0.0,0.5*r))
    else:
        r = random.uniform(0.0,r)
    a = random.uniform(0.0,2*math.pi)
    x += math.cos(a)*r
    y += math.sin(a)*r
    return [x, y]

class RadiusRandomize(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-r", "--radius",
                        action="store", type="float", 
                        dest="radius", default=10.0,
                        help="Randomly move control and end points in this radius")
        self.OptionParser.add_option("-c", "--ctrl",
                        action="store", type="inkbool", 
                        dest="ctrl", default=True,
                        help="Randomize control points")
        self.OptionParser.add_option("-e", "--end",
                        action="store", type="inkbool", 
                        dest="end", default=True,
                        help="Randomize nodes")
        self.OptionParser.add_option("-n", "--norm",
                        action="store", type="inkbool", 
                        dest="norm", default=True,
                        help="Use normal distribution")
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                d = node.attributes.getNamedItem('d')
                p = cubicsuperpath.parsePath(d.value)
                for subpath in p:
                    for csp in subpath:
                        if self.options.end:
                            delta=randomize([0,0], self.options.radius, self.options.norm)
                            csp[0][0]+=delta[0] 
                            csp[0][1]+=delta[1] 
                            csp[1][0]+=delta[0] 
                            csp[1][1]+=delta[1] 
                            csp[2][0]+=delta[0] 
                            csp[2][1]+=delta[1] 
                        if self.options.ctrl:
                            csp[0]=randomize(csp[0], self.options.radius, self.options.norm)
                            csp[2]=randomize(csp[2], self.options.radius, self.options.norm)
                d.value = cubicsuperpath.formatPath(p)

e = RadiusRandomize()
e.affect()




#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex, simplestyle, simplepath

class Dots(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-d", "--dotsize",
                        action="store", type="string", 
                        dest="dotsize", default="10px",
                        help="Size of the dots placed at path nodes")
        self.OptionParser.add_option("-f", "--fontsize",
                        action="store", type="string", 
                        dest="fontsize", default="20",
                        help="Size of node label numbers")    
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                self.group = self.document.createElement('svg:g')
                node.parentNode.appendChild(self.group)
                new = self.document.createElement('svg:path')
                
                try:
                    t = node.attributes.getNamedItem('transform').value
                    self.group.setAttribute('transform', t)
                except AttributeError:
                    pass

                s = simplestyle.parseStyle(node.attributes.getNamedItem('style').value)
                s['stroke-linecap']='round'
                s['stroke-width']=self.options.dotsize
                new.setAttribute('style', simplestyle.formatStyle(s))

                a =[]
                p = simplepath.parsePath(node.attributes.getNamedItem('d').value)
                num = 1
                for cmd,params in p:
                    if cmd != 'Z':
                        a.append(['M',params[-2:]])
                        a.append(['L',params[-2:]])
                        self.addText(self.group,params[-2],params[-1],num)
                        num += 1
                new.setAttribute('d', simplepath.formatPath(a))
                self.group.appendChild(new)
                node.parentNode.removeChild(node)

                
    def addText(self,node,x,y,text):
                new = self.document.createElement('svg:text')
                s = {'font-size': self.options.fontsize, 'fill-opacity': '1.0', 'stroke': 'none',
                    'font-weight': 'normal', 'font-style': 'normal', 'fill': '#000000'}
                new.setAttribute('style', simplestyle.formatStyle(s))
                new.setAttribute('x', str(x))
                new.setAttribute('y', str(y))
                new.appendChild(self.document.createTextNode(str(text)))
                node.appendChild(new)

e = Dots()
e.affect()



#!/usr/bin/env python
# -*- coding: cp1252 -*-
"""
EQTEXSVG.py
functions for converting LATEX equation string into SVG path
This extension need, to work properly :
    - a TEX/LATEX distribution (MiKTEX ...)
    - pstoedit software: <http://www.pstoedit.net/pstoedit>

Copyright (C) 2006 Julien Vitard, julienvitard@gmail.com

- I will try to code XML parsing, not the hard way ;-)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

import inkex, os, tempfile

def create_equation_tex(filename, equation):
    tex = open(filename, 'w')
    tex.write("""%% processed with EqTeXSVG.py
\documentclass{article}
    
\\thispagestyle{empty}
\\begin{document}
""")
    tex.write("$$\n")
    tex.write(equation)
    tex.write("\n$$\n")
    tex.write("\end{document}\n")
    tex.close()

def svg_open(self,filename):
    # parsing of SVG file with the equation 
    # real parsing XML to use!!!! it will be easier !!!
    svg = open(filename, 'r')
    svg_lines = svg.readlines()
    
    # trip top/bottom lines from svg file
    svg_lines.pop(0)
    svg_lines.pop(1)
    svg_lines.pop(len(svg_lines)-1)

    group = self.document.createElement('svg:g')
    self.current_layer.appendChild(group)

    # deleting "<g... >" "</g>" "<path d=" and "/>" from svg_lines
    nodegroup=''
    s_nodegroup_path=''
        
    for i in range(1,len(svg_lines)):
        if svg_lines[i].find("<g") != -1:
            nodegroup=svg_lines[i].split("<g")
            nodegroup=nodegroup[1].split(" >")
            nodegroup=nodegroup[0]+'\n'
        elif svg_lines[i].find("<path d=") != -1:
            s_nodegroup_path=svg_lines[i].split("<path d=")
            s_nodegroup_path=s_nodegroup_path[1]                
        elif svg_lines[i].find("/>") != -1:
            s_nodegroup_path=s_nodegroup_path+'"\n'
        elif svg_lines[i].find("</g>") != -1:
            nodegroup_svg = self.document.createElement('svg:g')
            nodegroup_svg.setAttribute('style',nodegroup)
            nodegroup_path = self.document.createElement('svg:path')
            nodegroup_path.setAttribute('d',s_nodegroup_path)
            group.appendChild(nodegroup_svg)                
            nodegroup_svg.appendChild(nodegroup_path)
        else:
            s_nodegroup_path=s_nodegroup_path+svg_lines[i]

class EQTEXSVG(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-f", "--formule",
                        action="store", type="string", 
                        dest="formule", default=10.0,
                        help="Formule LaTeX")
    def effect(self):
        
        base_file = os.path.join(tempfile.gettempdir(), "inkscape-latex.tmp")
        latex_file = base_file + ".tex"
        create_equation_tex(latex_file, self.options.formule)

        out_file = os.path.join(tempfile.gettempdir(), "inkscape-latex.tmp.output")
        os.system('latex -output-directory=' + tempfile.gettempdir() + ' ' + latex_file + '> ' + out_file)

        ps_file = base_file + ".ps"
        dvi_file = base_file + ".dvi"
        svg_file = base_file + ".svg"
        os.system('dvips -q -f -E -D 600 -y 5000 -o ' + ps_file + ' ' + dvi_file)
        os.system('pstoedit -f svg -dt -ssp ' + ps_file + ' ' + svg_file + '>> ' + out_file)

        # ouvrir le svg et remplacer #7F7F7F par #000000
        svg_open(self, svg_file)

        # clean up
        aux_file = base_file + ".aux"
        log_file = base_file + ".log"
        os.remove(latex_file)
        os.remove(aux_file)
        os.remove(log_file)
        os.remove(dvi_file)
        os.remove(ps_file)
        os.remove(svg_file)
        os.remove(out_file)
        
e = EQTEXSVG()
e.affect()



#!/usr/bin/env python
"""
inkex.py
A helper module for creating Inkscape extensions

Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
import sys, copy, optparse

#a dictionary of all of the xmlns prefixes in a standard inkscape doc
NSS = {
u'sodipodi'    :u'http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd',
u'cc'        :u'http://web.resource.org/cc/',
u'svg'        :u'http://www.w3.org/2000/svg',
u'dc'        :u'http://purl.org/dc/elements/1.1/',
u'rdf'        :u'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
u'inkscape'    :u'http://www.inkscape.org/namespaces/inkscape',
u'xlink'    :u'http://www.w3.org/1999/xlink'
}

try:
    import xml.dom.ext
    import xml.dom.ext.reader.Sax2
    import xml.xpath
except:
    sys.exit('The inkex.py module requires PyXML. Please download the latest version from <http://pyxml.sourceforge.net/>.')

def debug(what):
    sys.stderr.write(str(what) + "\n")
    return what

def check_inkbool(option, opt, value):
    if str(value).capitalize() == 'True':
        return True
    elif str(value).capitalize() == 'False':
        return False
    else:
        raise OptionValueError("option %s: invalid inkbool value: %s" % (opt, value))

class InkOption(optparse.Option):
    TYPES = optparse.Option.TYPES + ("inkbool",)
    TYPE_CHECKER = copy.copy(optparse.Option.TYPE_CHECKER)
    TYPE_CHECKER["inkbool"] = check_inkbool


class Effect:
    """A class for creating Inkscape SVG Effects"""
    def __init__(self):
        self.document=None
        self.selected={}
        self.options=None
        self.args=None
        self.OptionParser = optparse.OptionParser(usage="usage: %prog [options] SVGfile",option_class=InkOption)
        self.OptionParser.add_option("--id",
                        action="append", type="string", dest="ids", default=[], 
                        help="id attribute of object to manipulate")
    def effect(self):
        pass
    def getoptions(self,args=sys.argv[1:]):
        """Collect command line arguments"""
        self.options, self.args = self.OptionParser.parse_args(args)
    def parse(self,file=None):
        """Parse document in specified file or on stdin"""
        reader = xml.dom.ext.reader.Sax2.Reader()
        try:
            try:
                stream = open(file,'r')
            except:
                stream = open(self.args[-1],'r')
        except:
            stream = sys.stdin
        self.document = reader.fromStream(stream)
        stream.close()
    def getposinlayer(self):
        ctx = xml.xpath.Context.Context(self.document,processorNss=NSS)
        #defaults
        self.current_layer = self.document.documentElement
        self.view_center = (0.0,0.0)

        layerattr = xml.xpath.Evaluate('//sodipodi:namedview/@inkscape:current-layer',self.document,context=ctx)
        if layerattr:
            layername = layerattr[0].value
            layer = xml.xpath.Evaluate('//g[@id="%s"]' % layername,self.document,context=ctx)
            if layer:
                self.current_layer = layer[0]

        xattr = xml.xpath.Evaluate('//sodipodi:namedview/@inkscape:cx',self.document,context=ctx)
        yattr = xml.xpath.Evaluate('//sodipodi:namedview/@inkscape:cy',self.document,context=ctx)
        if xattr and yattr:
            x = xattr[0].value
            y = yattr[0].value
            if x and y:
                self.view_center = (float(x),float(y))
    def getselected(self):
        """Collect selected nodes"""
        for id in self.options.ids:
            path = '//*[@id="%s"]' % id
            for node in xml.xpath.Evaluate(path,self.document):
                self.selected[id] = node
    def output(self):
        """Serialize document into XML on stdout"""
        xml.dom.ext.Print(self.document)
    def affect(self):
        """Affect an SVG document with a callback effect"""
        self.getoptions()
        self.parse()
        self.getposinlayer()
        self.getselected()
        self.effect()
        self.output()



#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

import inkex, base64

class MyEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("--filepath",
                        action="store", type="string", 
                        dest="filepath", default=None,
                        help="")
    def effect(self):
        ctx = inkex.xml.xpath.Context.Context(self.document,processorNss=inkex.NSS)
        
        # exbed the first embedded image
        path = self.options.filepath
        if (path != ''):
            if (self.options.ids):
                for id, node in self.selected.iteritems():
                    if node.tagName == 'image':
                        xlink = node.attributes.getNamedItemNS(inkex.NSS[u'xlink'],'href')
                        if (xlink.value[:4]=='data'):
                            comma = xlink.value.find(',')
                            if comma>0:
                                data = base64.decodestring(xlink.value[comma:])
                                open(path,'wb').write(data)
                                xlink.value = path
                            else:
                                inkex.debug('Difficulty finding the image data.')
                            break

e = MyEffect()
e.affect()



#!/usr/bin/env python
from bezmisc import *
from ffgeom import *

def maxdist(((p0x,p0y),(p1x,p1y),(p2x,p2y),(p3x,p3y))):
    p0 = Point(p0x,p0y)
    p1 = Point(p1x,p1y)
    p2 = Point(p2x,p2y)
    p3 = Point(p3x,p3y)

    s1 = Segment(p0,p3)
    return max(s1.distanceToPoint(p1),s1.distanceToPoint(p2))
    

def cspsubdiv(csp,flat):
    for sp in csp:
        subdiv(sp,flat)

def subdiv(sp,flat,i=1):
    p0 = sp[i-1][1]
    p1 = sp[i-1][2]
    p2 = sp[i][0]
    p3 = sp[i][1]
    
    b = (p0,p1,p2,p3)
    m = maxdist(b)
    if m <= flat:
        try:
            subdiv(sp,flat,i+1)
        except IndexError:
            pass
    else:
        one, two = beziersplitatt(b,0.5)
        sp[i-1][2] = one[1]
        sp[i][0] = two[2]
        p = [one[2],one[3],two[1]]
        sp[i:1] = [p]    
        subdiv(sp,flat,i)


#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex, cubicsuperpath, simplestyle, copy, math, re, bezmisc

def numsegs(csp):
    return sum([len(p)-1 for p in csp])
def tpoint((x1,y1), (x2,y2), t = 0.5):
    return [x1+t*(x2-x1),y1+t*(y2-y1)]
def cspbezsplit(sp1, sp2, t = 0.5):
    m1=tpoint(sp1[1],sp1[2],t)
    m2=tpoint(sp1[2],sp2[0],t)
    m3=tpoint(sp2[0],sp2[1],t)
    m4=tpoint(m1,m2,t)
    m5=tpoint(m2,m3,t)
    m=tpoint(m4,m5,t)
    return [[sp1[0][:],sp1[1][:],m1], [m4,m,m5], [m3,sp2[1][:],sp2[2][:]]]
def cspbezsplitatlength(sp1, sp2, l = 0.5, tolerance = 0.001):
    bez = (sp1[1][:],sp1[2][:],sp2[0][:],sp2[1][:])
    t = bezmisc.beziertatlength(bez, l, tolerance)
    return cspbezsplit(sp1, sp2, t)
def cspseglength(sp1,sp2, tolerance = 0.001):
    bez = (sp1[1][:],sp1[2][:],sp2[0][:],sp2[1][:])
    return bezmisc.bezierlength(bez, tolerance)    
def csplength(csp):
    total = 0
    lengths = []
    for sp in csp:
        lengths.append([])
        for i in xrange(1,len(sp)):
            l = cspseglength(sp[i-1],sp[i])
            lengths[-1].append(l)
            total += l            
    return lengths, total
def numlengths(csplen):
    retval = 0
    for sp in csplen:
        for l in sp:
            if l > 0:
                retval += 1
    return retval

class SplitIt(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-m", "--max",
                        action="store", type="float", 
                        dest="max", default=0.0,
                        help="maximum segment length")
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                d = node.attributes.getNamedItem('d')
                p = cubicsuperpath.parsePath(d.value)
                
                #lens, total = csplength(p)
                #avg = total/numlengths(lens)
                #inkex.debug("average segment length: %s" % avg)

                new = []
                for sub in p:
                    new.append([sub[0][:]])
                    i = 1
                    while i <= len(sub)-1:
                        length = cspseglength(new[-1][-1], sub[i])
                        if length > self.options.max:
                            splits = math.ceil(length/self.options.max)
                            for s in xrange(int(splits),1,-1):
                                new[-1][-1], next, sub[i] = cspbezsplitatlength(new[-1][-1], sub[i], 1.0/s)
                                new[-1].append(next[:])
                        new[-1].append(sub[i])
                        i+=1
                    
                d.value = cubicsuperpath.formatPath(new)

e = SplitIt()
e.affect()



#!/usr/bin/env python 
'''
Copyright (C) 2006 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex, cubicsuperpath, simplepath, cspsubdiv

class MyEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-f", "--flatness",
                        action="store", type="float", 
                        dest="flat", default=10.0,
                        help="Minimum flatness of the subdivided curves")
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                d = node.attributes.getNamedItem('d')
                p = cubicsuperpath.parsePath(d.value)
                cspsubdiv.cspsubdiv(p, self.options.flat)
                np = []
                for sp in p:
                    first = True
                    for csp in sp:
                        cmd = 'L'
                        if first:
                            cmd = 'M'
                        first = False
                        np.append([cmd,[csp[1][0],csp[1][1]]])
                        d.value = simplepath.formatPath(np)

e = MyEffect()
e.affect()


#!/usr/bin/env python
"""
cubicsuperpath.py

Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
import simplepath 

def CubicSuperPath(simplepath):
    csp = []
    subpath = -1
    subpathstart = []
    last = []
    lastctrl = []
    for s in simplepath:
        cmd, params = s        
        if cmd == 'M':
            if last:
                csp[subpath].append([lastctrl[:],last[:],last[:]])
            subpath += 1
            csp.append([])
            subpathstart =  params[:]
            last = params[:]
            lastctrl = params[:]
        elif cmd == 'L':
            csp[subpath].append([lastctrl[:],last[:],last[:]])
            last = params[:]
            lastctrl = params[:]
        elif cmd == 'C':
            csp[subpath].append([lastctrl[:],last[:],params[:2]])
            last = params[-2:]
            lastctrl = params[2:4]
        elif cmd == 'Q':
            #TODO: convert to cubic
            csp[subpath].append([lastctrl[:],last[:],last[:]])
            last = params[-2:]
            lastctrl = params[-2:]
        elif cmd == 'A':
            #TODO: convert to cubics
            csp[subpath].append([lastctrl[:],last[:],last[:]])
            last = params[-2:]
            lastctrl = params[-2:]
        elif cmd == 'Z':
            csp[subpath].append([lastctrl[:],last[:],last[:]])
            last = subpathstart[:]
            lastctrl = subpathstart[:]
    #append final superpoint
    csp[subpath].append([lastctrl[:],last[:],last[:]])
    return csp    

def unCubicSuperPath(csp):
    a = []
    for subpath in csp:
        if subpath:
            a.append(['M',subpath[0][1][:]])
            for i in range(1,len(subpath)):
                a.append(['C',subpath[i-1][2][:] + subpath[i][0][:] + subpath[i][1][:]])
    return a

def parsePath(d):
    return CubicSuperPath(simplepath.parsePath(d))

def formatPath(p):
    return simplepath.formatPath(unCubicSuperPath(p))







#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex, simplepath, cubicsuperpath, re

uuconv = {'in':90.0, 'pt':1.25, 'px':1, 'mm':3.5433070866, 'cm':35.433070866, 'pc':15.0}
def unittouu(string):
    unit = re.compile('(%s)$' % '|'.join(uuconv.keys()))
    param = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')

    p = param.match(string)
    u = unit.search(string)    
    if p:
        retval = float(p.string[p.start():p.end()])
    else:
        retval = 0.0
    if u:
        try:
            return retval * uuconv[u.string[u.start():u.end()]]
        except KeyError:
            pass
    return retval

class MyEffect(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.dxf = ''
    def output(self):
        print self.dxf
    def dxf_add(self, str):
        self.dxf += str
    def dxf_line(self,csp):
        line = "\n0\nLINE\n8\n2\n62\n4\n10\n%f\n20\n%f\n30\n0\n11\n%f\n21\n%f\n31\n0" % (csp[0][0],csp[0][1],csp[1][0],csp[1][1])
        self.dxf_add(line)
    def dxf_spline(self,csp):
        knots = 8
        ctrls = 4
        self.dxf_add("\n  0\nSPLINE\n  5\n43\n  8\n0\n 62\n256\n370\n-1\n  6\nByLayer")
        self.dxf_add("\n100\nAcDbEntity\n100\nAcDbSpline\n 70\n8\n 71\n3\n 72\n%d\n 73\n%d\n 74\n0" % (knots, ctrls))
        for i in range(2):
            for j in range(4): 
                self.dxf_add("\n 40\n%d" % i)
        for i in csp:
            self.dxf_add("\n 10\n%f\n 20\n%f\n 30\n0" % (i[0],i[1]))
    def effect(self):
        #References:   Minimum Requirements for Creating a DXF File of a 3D Model By Paul Bourke
        #              NURB Curves: A Guide for the Uninitiated By Philip J. Schneider
        self.dxf_add("999\nDXF created by Inkscape\n0\nSECTION\n2\nENTITIES")
        
        scale = 25.4/90.0
        h = unittouu(inkex.xml.xpath.Evaluate('/svg/@height',self.document)[0].value)

        path = '//path'
        for node in inkex.xml.xpath.Evaluate(path,self.document):
            d = node.attributes.getNamedItem('d').value
            sim = simplepath.parsePath(d)
            simplepath.scalePath(sim,scale,-scale)
            simplepath.translatePath(sim,0,h*scale)            
            p = cubicsuperpath.CubicSuperPath(sim)
            for sub in p:
                for i in range(len(sub)-1):
                    s = sub[i]
                    e = sub[i+1]
                    if s[1] == s[2] and e[0] == e[1]:
                        self.dxf_line([s[1],e[1]])
                    else:
                        self.dxf_spline([s[1],s[2],e[0],e[1]])
        self.dxf_add("\n0\nENDSEC\n0\nEOF\n")


e = MyEffect()
e.affect()



#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import math, tempfile, cPickle, inkex, simplepath

def findend(d):
    end = []
    subPathStart = []
    for cmd,params in d:
        if cmd == 'M':
            subPathStart = params[-2:]
        if cmd == 'Z':
            end = subPathStart[:]
        else:
            end = params[-2:]
    return end

class LoadKochify(inkex.Effect):
    def effect(self):
        for id, node in self.selected.iteritems():
            if node.tagName == 'path':
                d = simplepath.parsePath(node.attributes.getNamedItem('d').value)
                start = d[0][1][-2:]
                end = findend(d)
                while start == end and len(d):
                    d = d[:-1]
                    end = findend(d)
                if not end: 
                    break
                dx = end[0]-start[0]
                dy = end[1]-start[1]
                length = math.sqrt((dx**2) + (dy**2))
                angle = math.atan2(dy,dx)
                endsinz = False
                if d[-1][0]=='Z':
                    endsinz = True
                path = {'start': start, 'end': end, 'endsinz': endsinz,
                    'length': length, 'angle': angle, 'path': d}
                f = open(tempfile.gettempdir() + '/kochify.bin', 'w')
                cPickle.dump(path, f)
                f.close()                
                break

e = LoadKochify()
e.affect()



#!/usr/bin/env python
"""
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
import inkex, os, re, simplepath, cubicsuperpath
from ffgeom import *

uuconv = {'in':90.0, 'pt':1.25, 'px':1, 'mm':3.5433070866, 'cm':35.433070866, 'pc':15.0}
def unittouu(string):
    unit = re.compile('(%s)$' % '|'.join(uuconv.keys()))
    param = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')

    p = param.match(string)
    u = unit.search(string)    
    if p:
        retval = float(p.string[p.start():p.end()])
    else:
        retval = 0.0
    if u:
        try:
            return retval * uuconv[u.string[u.start():u.end()]]
        except KeyError:
            pass
    return retval

class Project(inkex.Effect):
    def __init__(self):
            inkex.Effect.__init__(self)
    def effect(self):
        if len(self.options.ids) < 2:
            inkex.debug("Requires two selected paths. The second must be exctly four nodes long.")
            exit()            
            
        #obj is selected second
        obj = self.selected[self.options.ids[0]]
        trafo = self.selected[self.options.ids[1]]
        if obj.tagName == 'path' and trafo.tagName == 'path':
            #distil trafo into four node points
            trafo = cubicsuperpath.parsePath(trafo.attributes.getNamedItem('d').value)
            trafo = [[Point(csp[1][0],csp[1][1]) for csp in subs] for subs in trafo][0][:4]

            #vectors pointing away from the trafo origin
            self.t1 = Segment(trafo[0],trafo[1])
            self.t2 = Segment(trafo[1],trafo[2])
            self.t3 = Segment(trafo[3],trafo[2])
            self.t4 = Segment(trafo[0],trafo[3])
    
            #query inkscape about the bounding box of obj
            self.q = {'x':0,'y':0,'width':0,'height':0}
            file = self.args[-1]
            id = self.options.ids[0]
            for query in self.q.keys():
                f = os.popen("inkscape --query-%s --query-id=%s %s" % (query,id,file))
                self.q[query] = float(f.read())
                f.close()
            #glean document height from the SVG
            docheight = unittouu(inkex.xml.xpath.Evaluate('/svg/@height',self.document)[0].value)
            #Flip inkscapes transposed renderer coords
            self.q['y'] = docheight - self.q['y'] - self.q['height']

            #process path
            d = obj.attributes.getNamedItem('d')
            p = cubicsuperpath.parsePath(d.value)
            for subs in p:
                for csp in subs:
                    csp[0] = self.trafopoint(csp[0])
                    csp[1] = self.trafopoint(csp[1])
                    csp[2] = self.trafopoint(csp[2])
            d.value = cubicsuperpath.formatPath(p)

    def trafopoint(self,(x,y)):
        #Transform algorithm thanks to Jose Hevia (freon)
        vector = Segment(Point(self.q['x'],self.q['y']),Point(x,y))
        xratio = abs(vector.delta_x())/self.q['width']
        yratio = abs(vector.delta_y())/self.q['height']
    
        horz = Segment(self.t1.pointAtRatio(xratio),self.t3.pointAtRatio(xratio))
        vert = Segment(self.t4.pointAtRatio(yratio),self.t2.pointAtRatio(yratio))

        p = intersectSegments(vert,horz)
        return [p['x'],p['y']]    

e = Project()
e.affect()


#!/usr/bin/env python 
'''
Copyright (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''
import inkex, simplestyle, pturtle, random

def stripme(s):
    return s.strip()	

class LSystem(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)
        self.OptionParser.add_option("-o", "--order",
                        action="store", type="int", 
                        dest="order", default=3,
                        help="number of iteration")
        self.OptionParser.add_option("-l", "--langle",
                        action="store", type="float", 
                        dest="langle", default=16.0,
                        help="angle for turning left")
        self.OptionParser.add_option("-r", "--rangle",
                        action="store", type="float", 
                        dest="rangle", default=16.0,
                        help="angle for turning right")
        self.OptionParser.add_option("-s", "--step",
                        action="store", type="float", 
                        dest="step", default=25.0,
                        help="step size")
        self.OptionParser.add_option("-p", "--randomizestep",
                        action="store", type="float", 
                        dest="randomizestep", default=0.0,
                        help="randomize step")
        self.OptionParser.add_option("-z", "--randomizeangle",
                        action="store", type="float", 
                        dest="randomizeangle", default=0.0,
                        help="randomize angle")
        self.OptionParser.add_option("-x", "--axiom",
                        action="store", type="string", 
                        dest="axiom", default="++F",
                        help="initial state of system")
        self.OptionParser.add_option("-u", "--rules",
                        action="store", type="string", 
                        dest="rules", default="F=FF-[-F+F+F]+[+F-F-F]",
                        help="replacement rules")
        self.stack = []
        self.turtle = pturtle.pTurtle()
    def iterate(self):
        self.rules = dict([map(stripme, i.split("=")) for i in self.options.rules.upper().split(";") if i.count("=")==1])
        string = self.__recurse(self.options.axiom.upper(),0)
        self.__compose_path(string)
        return self.turtle.getPath()
    def __compose_path(self, string):
        self.turtle.pu()
        self.turtle.setpos(self.view_center)
        self.turtle.pd()
        for c in string:
            if c in 'ABCDEF':
                self.turtle.pd()
                self.turtle.fd(self.options.step * (random.normalvariate(1.0, 0.01 * self.options.randomizestep)))
            elif c in 'GHIJKL':
                self.turtle.pu()
                self.turtle.fd(self.options.step * (random.normalvariate(1.0,  0.01 * self.options.randomizestep)))
            elif c == '+':
                self.turtle.lt(self.options.langle * (random.normalvariate(1.0,  0.01 * self.options.randomizeangle)))
            elif c == '-':
                self.turtle.rt(self.options.rangle * (random.normalvariate(1.0,  0.01 * self.options.randomizeangle)))
            elif c == '|':
                self.turtle.lt(180)
            elif c == '[':
                self.stack.append([self.turtle.getpos(), self.turtle.getheading()])
            elif c == ']':
                self.turtle.pu()
                pos,heading = self.stack.pop()
                self.turtle.setpos(pos)
                self.turtle.setheading(heading)

    def __recurse(self,rule,level):
        level_string = ''
        for c in rule:
            if level < self.options.order:
                try:
                    level_string = level_string + self.__recurse(self.rules[c],level+1)
                except KeyError:
                    level_string = level_string + c
            else:
                level_string = level_string + c 
        return level_string
            
    def effect(self):
        new = self.document.createElement('svg:path')
        s = {'stroke-linejoin': 'miter', 'stroke-width': '1.0px', 
            'stroke-opacity': '1.0', 'fill-opacity': '1.0', 
            'stroke': '#000000', 'stroke-linecap': 'butt', 
            'fill': 'none'}
        new.setAttribute('style', simplestyle.formatStyle(s))
        new.setAttribute('d', self.iterate())
        self.current_layer.appendChild(new)

e = LSystem()
e.affect()

